// -------------------------------------------------------------------------------------
// ScrollText.h
// Author: Martin D. Flynn, NeXT Computer, Inc.
// Description:
// This class provide a full-featured set of utilities to handle printing to a Text
// object within a ScrollView.  This includes the ability to change the text font and 
// color of the text printed to the Text object.  This class will also handle text sent
// to it from any secondary thread.  It will automatically send the data to be printed 
// to the main thread to be printed into the Text object (only if objectThreadPerform
// is compiled with the application).
// -------------------------------------------------------------------------------------
// Permission is granted to freely redistribute this source code, and to use fragments
// of this code in your own applications if you find them to be useful.  This class,
// along with the source code, come with no warranty of any kind, and the user assumes
// all responsibility for its use.
// -------------------------------------------------------------------------------------
// example:
//
//   /* scrollview outlet */
//   - setMyScrollView:anObject
//     {
//     myTextView = [ScrollText newScrollText:anObject];
//     [myTextView setAutoLineFeed:NO]; // default
//     return self;
//     }
//
//   /* display formatted error message */
//   - displayError:(char *)textError
//     {
//     textPrintf(myTextView, "ERROR: %s\n", textError);
//     return self;
//     }
//
// -------------------------------------------------------------------------------------
#import <objc/Object.h>

// --------------------------------------------------------------------------------
// formatted text size limit for 'textPrintf'
#define         textStringSIZE          1024    // actual size cannot be known

// --------------------------------------------------------------------------------
// method name synonyms
#define         setAutoLF               setAutoLineFeed

// -------------------------------------------------------------------------------------
// structures used internal to ScrollText

/* text attributes */
typedef struct textAttr_s {
    id                  fontId;
    int                 colorMode;      // 0=none, 1=gray, 2=color
    NXColor             color;          // contains only gray if mode=1
} textAttr_t;

// scroll text data queue
typedef struct textQueue_s {
    char                *record;
    textAttr_t          attr;
    void                *next;
} textQueue_t;

// --------------------------------------------------------------------------------
@interface ScrollText : Object
{

    id                  scrollView;
    id                  textView;
    
    textAttr_t          runAttr;                    // newly added text attributes
  
    BOOL                autoLf;                     // auto linefeed mode
  
    mutex_t             queueMutex;                 // text queue mutex
    textQueue_t         *queueData;                 // pointer to front of queue
    textQueue_t         *queueBack;                 // pointer to end of queue

}

// --------------------------------------------------------------------------------
+ newScrollText:anObject;
//  Creates a new ScrollText object to handle text scrolling.  anObject must be
//  a ScrollView object which has a Text content view.  This is compatible
//  with the outlet provided by the text scroll view object in Interface Builder.
//
// --------------------------------------------------------------------------------
- setAutoLineFeed:(BOOL)mode;
//  Sets the autoLineFeed options.  If set to YES, then a newLine will be sent
//  after each string written to the scroll text.  The default is NO. 
//
// --------------------------------------------------------------------------------
- docView;
//  Returns the ScrollView docView.
//
// --------------------------------------------------------------------------------
- scrollView;
//  Returns the ScrollView id.
//
// --------------------------------------------------------------------------------
- setTextAttributeFont:fontId;
- setTextAttributeGray:(float)aGray;
- setTextAttributeColor:(NXColor)aColor;
//  Set RTF font/gray run attributes for newly added text.
//
// --------------------------------------------------------------------------------
- clearScrollText;
//  This clears the contents of the Text ScrollView.  Returns self.
//
// --------------------------------------------------------------------------------
- deleteLinesFrom:(int)fLine to:(int)tLine;
//  This deletes the specified lines from the Text ScrollView. 
//
// --------------------------------------------------------------------------------
- (int)textLines;
//  Returns the current number of lines present in the Text object.
//
// --------------------------------------------------------------------------------
- print:sender;
//  Prints the contents of the ScrollView to the printer.
//
// --------------------------------------------------------------------------------
- (int)textPrintf:(const char*)fmt args:(va_list)args;
- (int)textPrintf:(const char*)fmt, ...;
int textPrintf(id textView, const char *fmt, ...);
//  Appends the specified formated string to the contents of the ScrollView.
//
// --------------------------------------------------------------------------------

@end
